@@ -11,10 +11,7 @@ import os
 import inspect
 from collections import OrderedDict
 
-import appdirs
-
 import tpDcc as tp
-from tpDcc import register
 from tpDcc.core import plugin, tool
 from tpDcc.libs.python import decorators, python, path as path_utils
 from tpDcc.libs.qt.core import contexts
@@ -37,6 +34,7 @@ class ToolsManager(plugin.PluginsManager, object):
         self._layout = dict()
         self._loaded_tools = dict()
         self._tools_to_load = OrderedDict()
+        self._hub_tools = list()
 
     # ============================================================================================================
     # BASE
@@ -59,6 +57,12 @@ class ToolsManager(plugin.PluginsManager, object):
             return False
 
         package_loader = pkg_loaders[0] if isinstance(pkg_loaders, (list, tuple)) else pkg_loaders
+        if not package_loader:
+            return False
+
+        if hasattr(package_loader, 'loader'):
+            if not package_loader.loader:
+                return False
 
         plugin_name = package_loader.filename if python.is_python2() else os.path.dirname(package_loader.loader.path)
         plugin_path = package_loader.fullname if python.is_python2() else package_loader.loader.name
@@ -480,17 +484,17 @@ class ToolsManager(plugin.PluginsManager, object):
         pkg_name = pkg_loader.filename if python.is_python2() else os.path.dirname(pkg_loader.loader.path)
         pkg_path = pkg_loader.fullname if python.is_python2() else pkg_loader.loader.name
 
-        if tool_config:
-            skip_modules = ['{}.{}'.format(pkg_path, mod) for mod in
-                            tool_config.data.get('skip_modules', list())]
-        else:
-            skip_modules = list()
-        importer.init_importer(pkg_path, skip_modules=skip_modules)
-
-        if dcc_loader:
-            dcc_skip_modules = ['{}.{}'.format(pkg_path, mod) for mod in
-                                dcc_config.data.get('skip_modules', list())]
-            importer.init_importer(pkg_path, skip_modules=dcc_skip_modules)
+        # if tool_config:
+        #     skip_modules = ['{}.{}'.format(pkg_path, mod) for mod in
+        #                     tool_config.data.get('skip_modules', list())]
+        # else:
+        #     skip_modules = list()
+        # importer.init_importer(pkg_path, skip_modules=skip_modules)
+        #
+        # if dcc_loader:
+        #     dcc_skip_modules = ['{}.{}'.format(pkg_path, mod) for mod in
+        #                         dcc_config.data.get('skip_modules', list())]
+        #     importer.init_importer(pkg_path, skip_modules=dcc_skip_modules)
 
         tool_found = None
         for sub_module in pkgutil.walk_packages([self._plugins[package_name][tool_to_run]['plugin_package_path']]):
@@ -512,7 +516,7 @@ class ToolsManager(plugin.PluginsManager, object):
         # if dcc_loader:
         #     tool_config = dcc_config
 
-        settings_path = appdirs.user_data_dir(appname=tool_id)
+        settings_path = path_utils.get_user_data_dir(appname=tool_id)
         settings_file = path_utils.clean_path(os.path.expandvars(os.path.join(settings_path, 'settings.cfg')))
         tool_settings = settings.QtSettings(filename=settings_file)
         if not tool_settings.has_setting('theme'):
@@ -545,8 +549,16 @@ class ToolsManager(plugin.PluginsManager, object):
         if not tool_inst:
             return None
 
-        self.close_tool(tool_id)
+        hub = kwargs.pop('hub', False)
+        if hub and tool_id != 'tpDcc-tools-hub':
+            hub_ui = self.get_last_focused_hub_ui(include_minimized=False)
+            if hub_ui:
+                hub_ui.toggle_toolset(tool_id)
+                return tool_inst
+            else:
+                tp.logger.warning('No HubUI tool opened. Opening tool using standard method ...')
 
+        self.close_tool(tool_id)
         with contexts.application():
             self._launch_tool(tool_inst, tool_id, *args, **kwargs)
             return tool_inst
@@ -571,7 +583,7 @@ class ToolsManager(plugin.PluginsManager, object):
         tool_to_close = self._loaded_tools[tool_id].attacher
         if not closed_tool and tool_to_close:
             tool_to_close.fade_close() if hasattr(tool_to_close, 'fade_close') else tool_to_close.close()
-        if force:
+        if force and tool_to_close:
             tool_to_close.setParent(None)
             tool_to_close.deleteLater()
         self._loaded_tools.pop(tool_id)
@@ -596,13 +608,65 @@ class ToolsManager(plugin.PluginsManager, object):
         :return: DccTool or None, executed tool instance
         """
 
-        self.close_tool(tool_id)
-        tool_inst._launch(*args, **kwargs)
-        self._loaded_tools[tool_id] = tool_inst
+        if tool_id == 'tpDcc-tools-hub':
+            tool_data = tool_inst._launch(*args, **kwargs)
+            tool_ui = tool_data['tool']
+            self._hub_tools.append(tool_ui)
+        else:
+            self.close_tool(tool_id)
+            tool_inst._launch(*args, **kwargs)
+            self._loaded_tools[tool_id] = tool_inst
+
         tp.logger.debug('Execution time: {}'.format(tool_inst.stats.execution_time))
 
         return tool_inst
 
+    # ============================================================================================================
+    # HUB
+    # ============================================================================================================
+
+    def close_hub_ui(self, hub_ui_inst):
+        if hub_ui_inst in self._hub_tools:
+            self._hub_tools.remove(hub_ui_inst)
+            tp.logger.debug('Close tpDcc Hub UI: {}'.format(hub_ui_inst))
+
+    def get_hub_uis(self):
+        return self._hub_tools
+
+    def get_last_focused_hub_ui(self, include_minimized=True):
+        """
+        Returns last focused Hub UI
+        :param include_minimized: bool, Whether or not take into consideration Hub UIs that are minimized
+        :return: HubUI
+        """
+
+        hub_ui_found = None
+        max_time = 0
+
+        all_hub_uis = self.get_hub_uis()
+        for ui in all_hub_uis:
+            if ui.isVisible() and ui.last_focused_time > max_time:
+                if (not include_minimized and not ui.isMinimized()) or include_minimized:
+                    hub_ui_found = ui
+                    max_time = ui.last_focused_time
+
+        return hub_ui_found
+
+    def get_last_opened_hub_ui(self):
+        """
+        Returns last opened Hub UI
+        :return: HubUI
+        """
+
+        hub_ui_found = None
+
+        all_hub_uis = self.get_hub_uis()
+        for ui in all_hub_uis:
+            if ui.isVisible():
+                hub_ui_found = ui
+
+        return hub_ui_found
+
     # ============================================================================================================
     # CONFIGS
     # ============================================================================================================
